@@ -1,16 +1,16 @@
+from __future__ import absolute_import
 import os
 import itertools
 
-from numpy.testing.utils import assert_equal, assert_raises
-from nose import with_setup, SkipTest
-from nose.plugins.attrib import attr
+from numpy.testing import assert_equal
+import pytest
 
 from brian2 import *
-from brian2.devices.device import reinit_devices
+from brian2.devices.device import reinit_and_delete
 from brian2.tests.utils import assert_allclose
 
 
-@attr('codegen-independent')
+@pytest.mark.codegen_independent
 def test_custom_events():
     # Set (could be moved in a setup)
     EL = -65*mV
@@ -31,8 +31,7 @@ def test_custom_events():
     # Event has size three now because there are three compartments
     assert_allclose(G.event_time1[:], [0.1, 0, 0]*ms)
 
-@attr('codegen-independent')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.codegen_independent
 def test_construction():
     BrianLogger.suppress_name('resolution_conflict')
     morpho = Soma(diameter=30*um)
@@ -49,8 +48,8 @@ def test_construction():
     '''
 
     # Check units of currents
-    assert_raises(DimensionMismatchError, lambda: SpatialNeuron(morphology=morpho,
-                                                                model=eqs))
+    with pytest.raises(DimensionMismatchError):
+        SpatialNeuron(morphology=morpho, model=eqs)
 
     eqs='''
     Im=gL*(EL-v) : amp/meter**2
@@ -86,8 +85,7 @@ def test_construction():
     assert_allclose(morpho.right.nextone.distance, 3*um + (0.5 + np.arange(3))*2./3.*um)
 
 
-@attr('codegen-independent')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.codegen_independent
 def test_construction_coordinates():
     # Same as test_construction, but uses coordinates instead of lengths to
     # set up everything
@@ -110,8 +108,8 @@ def test_construction_coordinates():
     '''
 
     # Check units of currents
-    assert_raises(DimensionMismatchError, lambda: SpatialNeuron(morphology=morpho,
-                                                                model=eqs))
+    with pytest.raises(DimensionMismatchError):
+        SpatialNeuron(morphology=morpho, model=eqs)
 
     eqs='''
     Im=gL*(EL-v) : amp/meter**2
@@ -152,8 +150,7 @@ def test_construction_coordinates():
     assert_allclose(morpho.right.nextone.distance, 3*um + (0.5 + np.arange(3))*2./3.*um)
 
 
-@attr('long')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.long
 def test_infinitecable():
     '''
     Test simulation of an infinite cable vs. theory for current pulse (Green function)
@@ -196,14 +193,13 @@ def test_infinitecable():
     theory = theory*1*nA*0.02*ms
     assert_allclose(v[t>0.5*ms],theory[t>0.5*ms], rtol=1e14, atol=1e10) # high error tolerance (not exact because not infinite cable)
 
-@attr('standalone-compatible')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
 def test_finitecable():
     '''
     Test simulation of short cylinder vs. theory for constant current.
     '''
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     BrianLogger.suppress_name('resolution_conflict')
 
     defaultclock.dt = 0.01*ms
@@ -239,14 +235,13 @@ def test_finitecable():
     theory = EL+ra*neuron.I[0]*cosh((length-x)/la)/sinh(length/la)
     assert_allclose(v-EL, theory-EL, rtol=1e12, atol=1e8)
 
-@attr('standalone-compatible')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
 def test_rallpack1():
     '''
     Rallpack 1
     '''
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     defaultclock.dt = 0.05*ms
 
     # Morphology
@@ -300,14 +295,13 @@ def test_rallpack1():
     assert 100*max_rel_x < 0.5
 
 
-@attr('standalone-compatible')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
 def test_rallpack2():
     '''
     Rallpack 2
     '''
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     defaultclock.dt = 0.1*ms
 
     # Morphology
@@ -384,14 +378,15 @@ def test_rallpack2():
     assert 100*max_rel_x < 0.5
 
 
-@attr('long', 'standalone-compatible')
-@with_setup(teardown=reinit_devices)
+
+@pytest.mark.standalone_compatible
+@pytest.mark.long
 def test_rallpack3():
     '''
     Rallpack 3
     '''
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     defaultclock.dt = 1*usecond
 
     # Morphology
@@ -466,14 +461,13 @@ def test_rallpack3():
     assert 100*max_rel_x < 0.5
 
 
-@attr('standalone-compatible')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
 def test_rall():
     '''
     Test simulation of a cylinder plus two branches, with diameters according to Rall's formula
     '''
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     BrianLogger.suppress_name('resolution_conflict')
 
     defaultclock.dt = 0.01*ms
@@ -537,8 +531,7 @@ def test_rall():
     v = neuron.R.v
     assert_allclose(v-EL, theory-EL, rtol=1e12, atol=1e8)
 
-@attr('standalone-compatible')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
 def test_basic_diffusion():
     # A very basic test that shows that propagation is working in a very basic
     # sense, testing all morphological classes
@@ -568,7 +561,7 @@ def test_basic_diffusion():
     assert all(abs(mon.v[:, -1]/mV + 10) < 0.25), mon.v[:, -1]/mV
 
 
-@attr('codegen-independent')
+@pytest.mark.codegen_independent
 def test_allowed_integration():
     morph = Soma(diameter=30 * um)
     EL = -70 * mV
@@ -624,9 +617,10 @@ def test_allowed_integration():
 
     for eqs in forbidden_eqs:
         # Should raise an error
-        assert_raises(TypeError, SpatialNeuron, morph, eqs)
+        with pytest.raises(TypeError):
+            SpatialNeuron(morph, eqs)
 
-@attr('codegen-independent')
+@pytest.mark.codegen_independent
 def test_spatialneuron_indexing():
     sec = Cylinder(length=50*um, diameter=10*um, n=1)
     sec.sec1 = Cylinder(length=50 * um, diameter=10 * um, n=2)
@@ -662,8 +656,15 @@ def test_spatialneuron_indexing():
     assert len(neuron.sec1.sec12.main.v[:]) == 8
     assert len(neuron.sec2.main.v[:]) == 16
     assert len(neuron.sec2.sec21.main.v[:]) == 32
+    # Accessing subgroups
+    assert len(neuron[0].indices[:]) == 1
+    assert len(neuron[0*um:50*um].indices[:]) == 1
+    assert len(neuron[0:1].indices[:]) == 1
+    assert len(neuron[sec.sec2.indices[:]]) == 16
+    assert len(neuron[sec.sec2]) == 16
 
-@attr('codegen-independent')
+
+@pytest.mark.codegen_independent
 def test_tree_index_consistency():
     # Test all possible trees with depth 3 and a maximum of 3 branches subtree
     # (a total of 84 trees)
@@ -713,7 +714,7 @@ def test_tree_index_consistency():
         # Separate subtrees should not overlap
         assert len(all_subsec_indices) == len(set(all_subsec_indices))
 
-@attr('codegen-independent')
+@pytest.mark.codegen_independent
 def test_spatialneuron_subtree_assignment():
     sec = Cylinder(length=50 * um, diameter=10 * um, n=2)
     sec.sec1 = Cylinder(length=50 * um, diameter=10 * um, n=2)
@@ -738,7 +739,7 @@ def test_spatialneuron_subtree_assignment():
     assert_allclose(neuron.sec2.sec21.v[:], np.ones(2)*5*volt)
 
 
-@attr('codegen-independent')
+@pytest.mark.codegen_independent
 def test_spatialneuron_morphology_assignment():
     sec = Cylinder(length=50 * um, diameter=10 * um, n=2)
     sec.sec1 = Cylinder(length=50 * um, diameter=10 * um, n=2)
@@ -764,11 +765,11 @@ def test_spatialneuron_morphology_assignment():
     assert_allclose(neuron.sec1.sec11.v[:], np.ones(2)*volt)
     assert_allclose(neuron.sec1.sec12.v[:], np.zeros(2)*volt)
 
-@attr('standalone-compatible', 'multiple-runs')
-@with_setup(teardown=reinit_devices)
+@pytest.mark.standalone_compatible
+@pytest.mark.multiple_runs
 def test_spatialneuron_capacitive_currents():
     if prefs.core.default_float_dtype is np.float32:
-        raise SkipTest('Need double precision for this test')
+        pytest.skip('Need double precision for this test')
     defaultclock.dt = 0.1*ms
     morpho = Cylinder(x=[0, 10]*cm, diameter=2*238*um, n=200, type='axon')
 
@@ -809,6 +810,7 @@ def test_spatialneuron_capacitive_currents():
     assert_allclose((mon.Im-mon.Ic).sum(axis=0)/(mA/cm**2), np.zeros(230),
                     atol=1e6)
 
+
 if __name__ == '__main__':
     test_custom_events()
     test_construction()
